@@ -15,6 +15,6 @@ data("dataNested")
 
 modNested <- frailtyPenal(Surv(t1, t2, event) ~ cluster(group) +
   subcluster(subgroup) + cov1 + cov2, data = dataNested,
-  n.knots = 8, kappa1 = 10000, cross.validation = TRUE)
+  n.knots = 8, kappa1 = 10000, cross.validation = TRUE, Frailty = TRUE)
 
 print(modNested, digits = 4)
\ No newline at end of file
